/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2003
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: dsk.c,v 1.11 2003/03/07 10:48:38 erik Rel $";
#endif
#include <stdlib.h>
#include <assert.h>
#include "monitor/debug.h"
#include "ui/resource.h"
#include "util.h"
#include "disk.h"
#include "fileio/dsk.h"
typedef struct _dskSectorInfo{uns8 track;uns8 head;uns8 sector;uns8 bps;uns8 st1;uns8 st2;uns8 unused[2];}dskSectorInfo;typedef struct _edskSectorInfo{uns8 track;uns8 head;uns8 sector;uns8 bps;uns8 st1;uns8 st2;uns8 sectorsize[2];}edskSectorInfo;typedef struct _dskTrackInfo{uns8 magic[13];uns8 unused[3];uns8 track;uns8 head;uns8 unused2[2];uns8 bps;uns8 spt;uns8 gap3;uns8 fill;dskSectorInfo si[29];}dskTrackInfo;typedef struct _edskTrackInfo{uns8 magic[13];uns8 unused[3];uns8 track;uns8 head;uns8 unused2[2];uns8 bps;uns8 spt;uns8 gap3;uns8 fill;edskSectorInfo si[29];}edskTrackInfo;typedef struct _dskDiskInfo{uns8 magic[34];uns8 creator[14];uns8 tracks;uns8 sides;uns8 tracksize[2];uns8 unused[204];}dskDiskInfo;typedef struct _edskDiskInfo{uns8 magic[34];uns8 creator[14];uns8 tracks;uns8 sides;uns8 unused[2];uns8 tracksizes[204];}edskDiskInfo;
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_DISK) != 0) { x } }
#else
#define DEB(x)
#endif
static DiskSectorInfo*dskGetSectorInfo(Disk*,unsigned int,unsigned int,unsigned int);Disk*DskOpen(const char*fname){Disk*dp;dp=Malloc(sizeof(Disk),"DskOpen");dp->readonly=0;if((dp->mmp=(uns8*)Mmap(fname,1,&(dp->readonly),&(dp->mmsize)))!=(uns8*)-1){dskDiskInfo*dip=(dskDiskInfo*)(dp->mmp);edskDiskInfo*edip=(edskDiskInfo*)(dp->mmp);unsigned int edsk=0;if(memcmp(dip->magic,"MV - CPC",8)!=0){if(memcmp(dip->magic,"EXTENDED",8)!=0){Msg(M_WARN,"disk image is not in DSK format");goto error;}edsk=1;dp->readonly=1;}dp->fname=fname;dp->numSides=dip->sides;assert(dp->numSides<=MAX_DISK_SIDES);dp->numTracks=dip->tracks;assert(dp->numSides<=MAX_DISK_TRACKS);CRITICAL_START();if(edsk==0){dskTrackInfo*tp=(dskTrackInfo*)(dp->mmp+sizeof(dskDiskInfo));unsigned int tracksize=dip->tracksize[0]+(dip->tracksize[1]<<8);unsigned int side,track,sector;DEB(Msg(M_DEBUG,"DSK: standard format"););for(track=0;track<dp->numTracks;track++){for(side=0;side<dp->numSides;side++){uns8 dummy,*sp=(uns8*)tp+sizeof(dskTrackInfo);assert(tp->spt<=MAX_DISK_SECTORS);dp->tracks[side][track].numSectors=tp->spt;for(sector=0;sector<tp->spt;sector++){unsigned int sectorsize=128U<<tp->si[sector].bps;dp->tracks[side][track].sectors[sector].size=sectorsize;dp->tracks[side][track].sectors[sector].info=(uns8*)&(tp->si[sector]);dp->tracks[side][track].sectors[sector].data=sp;dummy+=*sp;sp+=sectorsize;}tp=(dskTrackInfo*)((uns8*)tp+tracksize);}}}else{edskTrackInfo*tp=(edskTrackInfo*)(dp->mmp+sizeof(edskDiskInfo));int idx=0;unsigned int side,track,sector;DEB(Msg(M_DEBUG,"DSK: extended format"););for(track=0;track<dp->numTracks;track++){for(side=0;side<dp->numSides;side++){assert(tp->spt<=MAX_DISK_SECTORS);if(edip->tracksizes[idx]!=0){uns8 dummy,*sp=(uns8*)tp+sizeof(dskTrackInfo);dp->tracks[side][track].numSectors=tp->spt;for(sector=0;sector<tp->spt;sector++){unsigned int sectorsize=tp->si[sector].sectorsize[0]+(tp->si[sector].sectorsize[1]<<8);dp->tracks[side][track].sectors[sector].size=sectorsize;dp->tracks[side][track].sectors[sector].info=(uns8*)&(tp->si[sector]);dp->tracks[side][track].sectors[sector].data=sp;dummy+=*sp;sp+=sectorsize;}}tp=(edskTrackInfo*)((uns8*)tp+(edip->tracksizes[idx++]<<8));}}}dp->getSectorInfo=dskGetSectorInfo;CRITICAL_ACTION(Msg(M_ERR,"corrupted input file"););CRITICAL_END();if(dp->getSectorInfo!=(DiskSectorInfo*(*)(struct _disk*,unsigned int,unsigned int,unsigned int))0){return dp;}error:Munmap(dp->mmp,dp->mmsize);}free(dp);return(Disk*)0;}static DiskSectorInfo*dskGetSectorInfo(Disk*dp,unsigned int side,unsigned int track,unsigned int sector){DiskSectorInfo*sip=(DiskSectorInfo*)0;if(side<dp->numSides&&track<dp->numTracks&&sector<dp->tracks[side][track].numSectors){sip=(DiskSectorInfo*)dp->tracks[side][track].sectors[sector].info;}return sip;}
