/*********************************************************************************************************************************
 * Copyright (c) Erik Kunze 1995 - 2003
 *
 * This file is a part of the Spectrum emulator package 'XZX-Pro', and may not be distributed.  The copyright holder makes no
 * representations about the suitability of this software for any purpose.  It is provided "as is" without express or implied
 * warranty. THE CODE MAY NOT BE MODIFIED OR REUSED WITHOUT PERMISSION!
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Erik Kunze
 ********************************************************************************************************************************/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifndef lint
static char rcsid[]="$Id: plus3.c,v 1.45 2003/03/26 23:07:40 erik Rel $";
#endif
#include <stdlib.h>
#include <assert.h>
#include "z80/z80.h"
#include "monitor/debug.h"
#include "ui/resource.h"
#include "machine/mem.h"
#include "machine/io.h"
#include "machine/machine.h"
#include "util.h"
#include "machine/cjoy.h"
#include "ddrive.h"
#include "emul.h"
#include "keyboard.h"
#include "machine/kjoy.h"
#include "ui/menu.h"
#ifdef XZX_MF
#include "machine/multiface.h"
#endif
#ifdef PRINTER
#include "printer/printer.h"
#endif
#include "ui/screen.h"
#include "machine/sjoy.h"
#include "sound/sound.h"
#include "tables.h"
#include "tape.h"
#include "ui/ui.h"
#include "machine/upd765a.h"
#include "machine/plus3.h"
#define Y_INTERRUPT	0
#define Y_RETRACE	15
#define Y_TOP_BORDER	48
#define Y_SCREEN	192
#define Y_BOTTOM_BORDER	56
#define X_LEFT_BORDER	24
#define X_SCREEN	128
#define X_RIGHT	24
#define X_RETRACE	48
#define ULA_DRAWS_SCREEN	0x04
#define ULA_DRAWS_BORDER	0x20
#define ULA_HORIZONTAL_RETRACE	0x40
#define ULA_VERTICAL_RETRACE	0x80
#define ULA_DELAY_MASK	0x0f
#define T_STATES_OVERFLOW	23 + 19 
#define IS_CONTENDED(addr)	(RPAGE((addr)) >= RAM4 )
#define P_BANK128	P_SP128_BANK
#define B_BANK128	B_SP128_BANK
#define B_SELROM	B_SP128_SELROM
#define B_SELRAM	B_SP128_SELRAM
#define B_SELSCREEN	B_SP128_SELSCREEN
#define PAGING_DISABLED	PAGING_SP128_DISABLED
#define B_TXD	0x08
#define B_DTR	0x40
#define B_SPECIAL	0x01	
#define B_SELRAMX	0x06	
#define B_SELROMX	0x04	
#define B_MOTOR	0x08	
#define B_STROBE	0x10	
#define EXTENDED_RAM(port)	(((port) & B_SPECIAL) != 0)
#define B_BUSY	0x01	
enum{ROM_UNKNOWN,ROM_PLUS3V40,ROM_PLUS3V41};
#ifdef DEBUG
#define DEB(x)	{ if ((CFGGET(debug) & D_IO) != 0) { x } }
#else
#define DEB(x)
#endif
static void plus3Reset(void);static void plus3Nmi(void);static uns8 plus3ReadMem(uns16);static void plus3WriteMem(uns16,uns8);static void plus3InitPages(void);static void plus3PageIn(int,int);static inline void plus3OutBankm128(uns8);static inline void plus3OutBank678(uns8);static void plus3OutPort(uns16,uns8);static uns8 plus3InPort(uns16);static Upd765aFdc*plus3Fdc;void Plus3Init(void){MachineSettings*msp;int checksum,romtype;unsigned int i,x,y;msp=&Machines[SP_3];msp->name="Spectrum +3";msp->basicRom=SP3ROM3;msp->tvLines=Y_RETRACE+Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;msp->tstatesPerLine=X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;msp->tstatesPerFrame=msp->tvLines*msp->tstatesPerLine;msp->intLine=Y_INTERRUPT;msp->topBorder=Y_RETRACE+Y_TOP_BORDER;msp->ulaDelay=Malloc(sizeof(unsigned char)*(size_t)(msp->tstatesPerFrame+T_STATES_OVERFLOW),"Plus3Init");msp->reset=plus3Reset;msp->nmi=plus3Nmi;msp->readOpcode=plus3ReadMem;msp->readMem=plus3ReadMem;msp->writeMem=plus3WriteMem;msp->inPort=plus3InPort;msp->outPort=plus3OutPort;msp->initPages=plus3InitPages;msp->pageIn=plus3PageIn;if(LoadRomImage(CFGGET(sp3Rom),SP3ROM0,4)==0){checksum=GetRomChecksum(msp->basicRom,XZX_PAGESIZE);switch(checksum){case 48934:romtype=ROM_PLUS3V40;break;case 10222:romtype=ROM_PLUS3V41;break;default:Msg(M_WARN,"unknown +3 ROM image - checksum %u",checksum);romtype=ROM_UNKNOWN;break;}if(romtype!=ROM_UNKNOWN){msp->traps.load.page=msp->basicRom;msp->traps.load.addr=0x563;msp->traps.save.page=msp->basicRom;msp->traps.save.addr=0x4c7;
#ifndef SOUND_NATIVE
msp->traps.beep.page=msp->basicRom;msp->traps.beep.addr=0x3b9;
#endif
}}for(i=0,y=0;y<Y_RETRACE;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER+Y_SCREEN;y++){for(x=0;x<X_LEFT_BORDER;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN;x++){msp->ulaDelay[i++]=ULA_DRAWS_SCREEN;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}for(;y<Y_RETRACE+Y_TOP_BORDER+Y_SCREEN+Y_BOTTOM_BORDER;y++){for(x=0;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT;x++){msp->ulaDelay[i++]=ULA_DRAWS_BORDER;}for(;x<X_LEFT_BORDER+X_SCREEN+X_RIGHT+X_RETRACE;x++){msp->ulaDelay[i++]=ULA_HORIZONTAL_RETRACE;}}while(i<msp->tstatesPerFrame+T_STATES_OVERFLOW){msp->ulaDelay[i++]=ULA_VERTICAL_RETRACE;}assert(i==msp->tstatesPerFrame+T_STATES_OVERFLOW);plus3Fdc=Upd765aNew(NELEM(CFGGET(sp3Disks)));for(i=0;i<NELEM(CFGGET(sp3Disks));i++){DdriveSetIF(plus3Fdc->drives[i],Plus3InsertDisk);if(DdriveInsertDisk(plus3Fdc->drives[i],CFGGET(sp3Disks[i]))<0){free(CFGGET(sp3Disks[i]));CFGSET(sp3Disks[i],(char*)0);}}}static void plus3Reset(void){Z80_Reset();Machine->initPages();Last0x7FFD =0;ScreenSelect=RAM5;plus3OutBank678(0);
#ifdef XZX_MF
Multiface3Paged=0;
#endif
Last0xFFFD =0;(void)memset(Psg,0x00,sizeof(Psg));
#ifdef SOUND_NATIVE
Ay8912Reset();
#endif
Upd765aReset(plus3Fdc);}static void plus3Nmi(void){Z80_NMI((NmiCallbackFunc)0);
#ifdef XZX_MF
(void)Machine->inPort(P_MF3_IN);
#endif
}static uns8 plus3ReadMem(uns16 addr){uns8 byte=MEM(addr);if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}return byte;}static void plus3WriteMem(uns16 addr,uns8 byte){if(!READ_ONLY(addr)){MEM(addr)=byte;if(IS_CONTENDED(addr)){TSTATES+=(unsigned int)Machine->ulaDelay[TSTATES]&ULA_DELAY_MASK;}if(RPAGE(addr)==ScreenSelect&&(addr&=0x3fff)<(uns16)(PIXEL_LENGTH+ATTR_LENGTH)){ScreenMarkDirty(addr);}}}static void plus3InitPages(void){Machine->pageIn(0,SP3ROM0);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM0);}static void plus3PageIn(int z80page,int vpage){int i=z80page<<1;TheMemory[i]=RealMemory[vpage];RealPageNumbers[i]=vpage;i++;TheMemory[i]=RealMemory[vpage]+XZX_PAGESIZE/2;switch(vpage){
#ifdef XZX_MF
case MF3ROM:RealPageNumbers[i]=MFRAM;break;
#endif
default:RealPageNumbers[i]=vpage;break;}}static inline void plus3OutBankm128(uns8 val){uns8 changed=val^Last0x7FFD;DEB(Msg(M_DEBUG,"bankm: RAM %d, ROM %d, screen %d, paging %sabled",val&B_SELRAM,(val&B_SELROM)>>4,(val&B_SELSCREEN)>>3,PAGING_DISABLED(Last0x7FFD)?"dis":"en"););if(changed==0||PAGING_DISABLED(Last0x7FFD)){return;}if((changed&B_SELSCREEN)!=0){ScreenSelect=val&B_SELSCREEN?RAM7:RAM5;ScreenMarkDirtyAll();}if(!EXTENDED_RAM(Last0x1FFD)){Machine->pageIn(3,RAM0+(val&B_SELRAM));
#ifdef XZX_MF
if(Multiface3Paged==0)
#endif
{Machine->pageIn(0,SP3ROM0+(((val&B_SELROM)>>4)|((Last0x1FFD &B_SELROMX)>>1)));}DEB(Msg(M_DEBUG,"bankm: horizontal ROM switch"););}Last0x7FFD =val;}static inline void plus3OutBank678(uns8 val){uns8 changed=val^Last0x1FFD;DEB(Msg(M_DEBUG,"bank678: %s",EXTENDED_RAM(val)?"extended RAM enabled":"vertical ROM switch"););if(changed==0||PAGING_DISABLED(Last0x7FFD)){return;}if(EXTENDED_RAM(val)){switch((val&B_SELRAMX)>>1){case 0:Machine->pageIn(0,RAM0);Machine->pageIn(1,RAM1);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM3);DEB(Msg(M_DEBUG,"bank678: 0,1,2,3"););break;case 1:Machine->pageIn(0,RAM4);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM6);Machine->pageIn(3,RAM7);DEB(Msg(M_DEBUG,"bank678: 4,5,6,7"););break;case 2:Machine->pageIn(0,RAM4);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM6);Machine->pageIn(3,RAM3);DEB(Msg(M_DEBUG,"bank678: 4,5,6,3"););break;case 3:Machine->pageIn(0,RAM4);Machine->pageIn(1,RAM7);Machine->pageIn(2,RAM6);Machine->pageIn(3,RAM3);DEB(Msg(M_DEBUG,"bank678: 4,7,6,3"););break;}}else{int page0;
#ifdef XZX_MF
if(Multiface3Paged){page0=MF3ROM;}else
#endif
{page0=SP3ROM0+(((Last0x7FFD &B_SELROM)>>4)|((val&B_SELROMX)>>1));}Machine->pageIn(0,page0);Machine->pageIn(1,RAM5);Machine->pageIn(2,RAM2);Machine->pageIn(3,RAM0+(Last0x7FFD &B_SELRAM));}if((changed&B_MOTOR)!=0){unsigned int state=(val&B_MOTOR)>>3;DdriveMotor(plus3Fdc->drives[0],state);DdriveMotor(plus3Fdc->drives[1],state);}
#ifdef PLUS3_PRINTER
if((changed&B_STROBE)!=0){CentronicsStrobe(val&B_STROBE);}
#endif
Last0x1FFD =val;}static void plus3OutPort(uns16 port,uns8 val){DEB(Msg(M_DEBUG,"writing <%02x> to port <%04x>",val,port););if((port&B_ULA)==(P_ULA&B_ULA)){uns8 changes=val^Last0xFE;if((changes&B_BORDER)!=0){ScreenSetBorder(val&B_BORDER);}
#ifdef SOUND_NATIVE
if((changes&(B_SPEAKER|B_MIC))!=0){SpeakerOutByte(val);}
#endif
Last0xFE =val;}if((port&B_128)==(P_128&B_128)){switch(port&0xc000){case P_SND_CONTROL&0xc000:Last0xFFFD =val;break;case P_SND_DATA&0xc000:if(Last0xFFFD <NELEM(Psg)){Psg[Last0xFFFD]=val;if(Last0xFFFD ==14){DEB(Msg(M_DEBUG,"writing AY register 14"););
#ifdef SP128_PRINTER
Sp128SerialOutByte(val&B_TXD);
#endif
}
#ifdef SOUND_NATIVE
else if(Last0xFFFD <14){Ay8912OutByte(Last0xFFFD,val);}
#endif
}else{DEB(Msg(M_DEBUG,"writing AY register %d",Last0xFFFD););}break;case P_BANK128&0xc000:plus3OutBankm128(val);break;default:switch(port&0x3000){
#ifdef PLUS3_PRINTER
case P_SP3_CENTRONICS&0x3000:CentronicsOutByte(val);break;
#endif
case P_SP3_BANK&0x3000:plus3OutBank678(val);break;case P_SP3_FDCCONTROL&0x3000:break;case P_SP3_FDCDATA&0x3000:Upd765aDataOutByte(plus3Fdc,val);break;}}}}static uns8 plus3InPort(uns16 port){uns8 res=0xff;int attached=0;if((port&B_ULA)==(P_ULA&B_ULA)){uns8 res2=0xbf;KEYBOARD_INBYTE(port,res2);SJOY_INBYTE(port,res2);CJOY_INBYTE(port,res2);res2^=TapeInGetBit();res&=res2;attached++;}if((port&B_128)==(P_128&B_128)){uns8 res2;switch(port&0xc000){case P_SND_CONTROL&0xc000:if(Last0xFFFD ==14){DEB(Msg(M_DEBUG,"reading AY register 14"););res2=0xbf;
#ifdef SP128_PRINTER
res2|=Sp128SerialStatus()!=0?B_DTR:0;
#endif
res2&=Psg[7]&0x40 ?Psg[14]:0xff;}else if(Last0xFFFD <14){res2=Psg[Last0xFFFD]&PsgMask[Last0xFFFD];}else if(Last0xFFFD ==15){res2=Psg[7]&0x80 ?Psg[15]:0xff;}else{res2=0x00;}res&=res2;attached++;break;case P_SND_DATA&0xc000:attached++;break;case P_BANK128&0xc000:attached++;break;default:switch(port&0x3000){
#ifdef PLUS3_PRINTER
case P_SP3_CENTRONICS&0x3000:res&=CentronicsStatus()!=0?0:B_BUSY;break;
#endif
case P_SP3_BANK&0x3000:break;case P_SP3_FDCCONTROL&0x3000:res&=Upd765aControlInByte(plus3Fdc);break;case P_SP3_FDCDATA&0x3000:res&=Upd765aDataInByte(plus3Fdc);break;}attached++;break;}}KJOY_INBYTE(port,res,attached);
#ifdef XZX_MF
MF3_INBYTE(port,res,attached);
#endif
DEB(Msg(M_DEBUG,"reading <%02x> from port <%04x>",res,port););return res;}int Plus3InsertDisk(int drive){int types[2]={FILE_TYPE_DSK,FILE_TYPE_END};char*fname;int ret=-1;if((fname=UiFileSelector("Insert Disk",1,types))!=(char*)0){if((ret=DdriveInsertDisk(plus3Fdc->drives[drive],fname))<0){free(fname);fname=(char*)0;}CFGSET(sp3Disks[drive],fname);MenuFileDiskUpdate();}return ret;}void Plus3EjectDisk(int drive){DdriveEjectDisk(plus3Fdc->drives[drive]);CFGSET(sp3Disks[drive],(char*)0);MenuFileDiskUpdate();}
